@@ -42,12 +42,18 @@ typedef enum {
 	MAPI_OPTIONS_DELETE_ON_SUBMIT_FAILURE = 1<<9
 } ExchangeMAPIOptions;
 
+/* Flags for push notification APIs*/
+typedef enum {
+	MAPI_EVENTS_USE_STORE = 1<<0,
+	MAPI_EVENTS_USE_PF_STORE = 1<<1,
+	MAPI_EVENTS_FOLDER = 1<<2,
+} ExchangeMAPIEventsOptions;
+
 #define MAPI_OPTIONS_FETCH_ALL MAPI_OPTIONS_FETCH_ATTACHMENTS | \
 	                       MAPI_OPTIONS_FETCH_RECIPIENTS | \
 	                       MAPI_OPTIONS_FETCH_BODY_STREAM | \
 	                       MAPI_OPTIONS_FETCH_GENERIC_STREAMS
 
-
 typedef struct {
 	GByteArray *value;
 	uint32_t proptag;
@@ -78,9 +84,12 @@ typedef struct {
 		struct SPropValue *ext_lpProps;
 	} in; 
 
-	/* These are properties which would be set on the 
-	 * recipients after GetRecipientTable() */
-	struct SRow out_SRow;
+	struct {
+		/* These are properties which would be set on the 
+		 * recipients after GetRecipientTable() */
+		uint32_t all_cValues; 
+		struct SPropValue *all_lpProps;
+	} out; 
 } ExchangeMAPIRecipient;
 
 typedef struct {
@@ -153,6 +162,9 @@ exchange_mapi_empty_folder (mapi_id_t fid);
 gboolean 
 exchange_mapi_rename_folder (mapi_id_t fid, const char *new_name);
 
+gboolean
+exchange_mapi_move_folder (mapi_id_t src_fid, mapi_id_t src_parent_fid, mapi_id_t des_fid, const char *new_name);
+
 GSList *
 exchange_mapi_util_check_restriction (mapi_id_t fid, struct mapi_SRestriction *res);
 
@@ -203,4 +215,24 @@ gboolean exchange_mapi_create_profile (const char *username, const char *passwor
 				       char **error_msg, mapi_profile_callback_t cb, gpointer data);
 gboolean exchange_mapi_delete_profile (const char *profile);
 
+
+/* Push notifications APIs */
+typedef gboolean (*exchange_check_continue) (void);
+
+gboolean exchange_mapi_events_init ();
+
+gboolean exchange_mapi_events_monitor (struct mapi_notify_continue_callback_data *cb_data);
+
+gboolean exchange_mapi_events_subscribe (guint32 options,
+					 guint16 event_mask, guint32 *connection,
+					 mapi_notify_callback_t callback, gpointer data);
+
+gboolean exchange_mapi_events_unsubscribe (guint32 connection);
+
+gboolean
+exchange_mapi_events_subscribe_and_monitor (mapi_id_t *obj_id, guint32 options,
+					    guint16 event_mask, guint32 *connection,
+					    gboolean use_store, mapi_notify_callback_t callback,
+					    gpointer data);
+
 #endif
